From fc25012559f28903d21d4722191e62f70f49aeee Mon Sep 17 00:00:00 2001
From: Sebastian Poeplau <sebastian.poeplau@eurecom.fr>
Date: Tue, 26 Mar 2019 13:45:57 +0100
Subject: [PATCH] Keep track of the time spent in each mode

The original logic maintaining those timers, activated with
"--enable-executor-timing", does not work; this is a workaround that updates the
timers correctly.
---
 include/s2e/S2EExecutor.h | 3 +++
 src/S2EExecutor.cpp       | 4 ++++
 2 files changed, 7 insertions(+)

diff --git a/include/s2e/S2EExecutor.h b/include/s2e/S2EExecutor.h
index 6ac68aa..7ad42f5 100644
--- a/include/s2e/S2EExecutor.h
+++ b/include/s2e/S2EExecutor.h
@@ -23,6 +23,7 @@ struct CPUX86State;
 
 namespace klee {
 struct Query;
+class TimerStatIncrementer;
 }
 
 namespace s2e {
@@ -79,6 +80,8 @@ protected:
 
     bool m_inLoadBalancing;
 
+    std::unique_ptr<klee::TimerStatIncrementer> m_symbolicModeTimerIncrementer;
+
     struct CPUTimer *m_stateSwitchTimer;
 
     /** Counts how many translation blocks reference a given LLVM function */
diff --git a/src/S2EExecutor.cpp b/src/S2EExecutor.cpp
index aed4846..d95386e 100644
--- a/src/S2EExecutor.cpp
+++ b/src/S2EExecutor.cpp
@@ -1252,6 +1252,8 @@ void S2EExecutor::switchToConcrete(S2EExecutionState *state) {
     state->m_registers.copySymbRegs(true);
 
     state->m_runningConcrete = true;
+
+    m_symbolicModeTimerIncrementer.reset(new klee::TimerStatIncrementer(stats::concreteModeTime));
 }
 
 void S2EExecutor::switchToSymbolic(S2EExecutionState *state) {
@@ -1270,6 +1272,8 @@ void S2EExecutor::switchToSymbolic(S2EExecutionState *state) {
 
     state->m_registers.copySymbRegs(false);
     state->m_runningConcrete = false;
+
+    m_symbolicModeTimerIncrementer.reset(new klee::TimerStatIncrementer(stats::symbolicModeTime));
 }
 
 void S2EExecutor::splitStates(const std::vector<S2EExecutionState *> &allStates, StateSet &parentSet,
-- 
2.21.0

